#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for Dixim DMS package."""

from iobase.exception import (InvalidArgError, IoBaseException, NotFoundError)
from ionas.request import (Entry,)

import libfal.falcmd as falcmd
import libfal.util as util
from fal_info import (ReqTypes, Results)
from pal_info import (PAL_CONFFILE_PATH, PAL_DIXIMDMS_PATH)

from fal_config import (SHARES_CONF_FILE,)
from pkgaddon.fal_diximdms_config import (CONFIG_FILE, DMS_SERVICE,)


DIXIMDMS_NAME = "diximdms"

# Add the service entry for the eset package
util.config.FUNC_TO_SERVICE_MAP[DMS_SERVICE] = DMS_SERVICE

SERVICE_NAME = "service"
APPLY_TAG_LIST = [SERVICE_NAME, "lang", "sharelist"]
DELETE_INFO_LIST_MAP = {
        "sharelist": [(".", "*")],
    }

SHARE_PATH = "sharelist/share"
NAME_NAME = "name"
SHARE_PATH_XPATH_TMPL = """sharelist/share/name[text()="%s"]/../path"""


class RequestError(IoBaseException):
    """Request error class."""

    pass


class FalDiximDms(falcmd.FalCmd):
    """FAL DiximDms command implementation class."""

    #
    # Internal methods
    #

    def _check_body_node(self, node, type=None):
        """Check if entry's body node is valid."""

        if node.get_node_name() != DIXIMDMS_NAME:
            raise InvalidArgError(node.get_node_name())

    def __send_to_pal(self, req_entry, pal=PAL_DIXIMDMS_PATH):
        """Send "req_entry" request to pal_eset."""

        return self.send_entry(req_entry, pal)

    def _send_request(self, target, entry):
        """Send "entry" request to "target" PAL commands."""

        response_entry = self.send_entry(entry, target)
        if response_entry.get_result() != Results.OK:
            raise RequestError(response_entry.get_result())

        return response_entry.get()

    def _try_deleting_old_setting(self, old_tag):
        """Try deleting old setting in "old_tag"."""

        del_info_list = DELETE_INFO_LIST_MAP.get(old_tag.get_node_name(), [])
        for del_info in del_info_list:
            old_tag.delete_nodes(del_info[0], del_info[1])

    def _get_pal_apply_entry(self, entry):
        """Return entry for "apply" request to pal."""

        req_entry = entry.get_clone()
        share_config = util.load_config(SHARES_CONF_FILE)
        for share in req_entry.get().get_node_list(SHARE_PATH):
            try:
                path = share_config.get_node(
                            SHARE_PATH_XPATH_TMPL % share.get_value(NAME_NAME))
                share.append_child(path.get_clone())
            except:
                # Ignore error
                pass

        return req_entry

    #
    # Request handlers
    #

    def _query_handler(self, entry):
        """Handle query request."""

        self._check_body_node(entry.get(), ReqTypes.QUERY)
        return self.__send_to_pal(Entry(ReqTypes.QUERY, DIXIMDMS_NAME))

    @falcmd.conf_change_handler_wrapper
    def _apply_handler(self, entry):
        """Handle apply request."""

        self._check_body_node(entry.get(), ReqTypes.APPLY)
        try:
            config = util.load_config(CONFIG_FILE)
            new_config = entry.get()

            is_service_enabled = None
            util.servicectrl_suspend([], [DMS_SERVICE])
            for tag_name in APPLY_TAG_LIST:
                try:
                    new_target = new_config.get_node(tag_name)
                    if tag_name == SERVICE_NAME:
                        is_service_enabled = new_target.is_enabled()

                    try:
                        target = config.get_node(tag_name)
                        self._try_deleting_old_setting(target)
                        target.update(new_target, override=True)
                    except NotFoundError:
                        config.append_child(new_target.get_clone())
                except:
                    # Not change setting about "tag_name" tag entry.
                    continue

            self._send_request(PAL_DIXIMDMS_PATH,
                                self._get_pal_apply_entry(entry))
            util.servicectrl_resume([], [DMS_SERVICE])

            if is_service_enabled is True:
                util.servicectrl_start([], [DMS_SERVICE])
            elif is_service_enabled is False:
                util.servicectrl_stop([], [DMS_SERVICE])

            util.save_config(CONFIG_FILE, config)
            entry.set_result(Results.OK)
        except:
            self.exception("apply fail")
            entry.set_result(Results.APPLY_ERROR)

        return entry

    def _initialize_handler(self, entry):
        """Handle list request."""

        self._check_body_node(entry.get())
        try:
            self._send_request(PAL_DIXIMDMS_PATH, entry)
            entry.set_result(Results.OK)
        except:
            self.exception("initialize fail")
            entry.set_result(Results.INITIALIZE_ERROR)
        return entry

    def _register_handler(self, entry):
        """Handle register request."""

        self._check_body_node(entry.get())
        try:
            util.servicectrl_stop([], [DMS_SERVICE])
            self._send_request(PAL_DIXIMDMS_PATH, entry)
            util.servicectrl_start([], [DMS_SERVICE])
            entry.set_result(Results.OK)
        except RequestError, err:
            self.exception("register fail(%s)" % err.get_info())
            entry.set_result(err.get_info())
        except:
            self.exception("register fail")
            entry.set_result(Results.INTERNAL_ERROR)
        return entry

    def _delete_handler(self, entry):
        """Handle delete request."""

        self._check_body_node(entry.get())
        try:
            self._send_request(PAL_DIXIMDMS_PATH, entry)
            entry.set_result(Results.OK)
        except:
            self.exception("delete fail")
            entry.set_result(Results.DELETE_ERROR)
        return entry

    #
    # Constructor
    #

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        load_save_handler = lambda entry: self.send_entry_with_file(
            entry, PAL_CONFFILE_PATH, CONFIG_FILE)

        self.set_handler(ReqTypes.LOAD, load_save_handler)
        self.set_handler(ReqTypes.SAVE, load_save_handler)

        self.set_handler(ReqTypes.QUERY, self._query_handler)
        self.set_handler(ReqTypes.APPLY, self._apply_handler)
        self.set_handler(ReqTypes.INITIALIZE, self._initialize_handler)
        self.set_handler(ReqTypes.REGISTER, self._register_handler)
        self.set_handler(ReqTypes.DELETE, self._delete_handler)


#
# Main
#

if __name__ == "__main__":
    FalDiximDms(option=falcmd.FalCmdOption()).handle_request()
