#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""Event handler to start or stop service for diximdms package."""

import os
import glob
import shutil

from iobase.xmldata import (XmlConfNode,)
import ionas.request as request
import evh_config as config
import libevh.evhcmd as evhcmd
from fal_info import (FAL_SERVICECTRL_PATH, FAL_DIXIMDMS_PATH, FAL_SHARES_PATH,
                        Results, ReqTypes)

from pkgaddon.diximdms_config import (DMS_SERVICE, DMS_DATABASE_SRC_PATH,
                                        DMS_DTCP_DATABASE_SRC_PATH,
                                        DMS_DATABASE_PATH,
                                        DMS_DTCP_PREPARE_DIR_LIST,
                                        DMS_DTCP_EXPORTS_PATH,)


EVENTINFO_NAME = "eventinfo"
VOLUME_NAME_PATH = "volume/name"
CATEGORY_ATTR = "category"
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"

VOLUME_CATEGORY = "volume"
SHARES_CATEGORY = "shares"
SYSTEM_CATEGORY = "system"
DIXIMDMS_CATEGORY = "diximdms"
MOUNT_ACTION = "mount"
UNMOUNT_ACTION = "unmount"
DELETE_ACTION = "delete"
SHUTDOWN_ACTION = "shutdown"
REBOOT_ACTION = "reboot"
APPLY_ACTION = "apply"

DIXIMDMS_SERVICE_PATH = "service"
SERVICECTRL_REQ_XML_TMPL = """<servicectrl>
    <servicelist>
        <service><name>%s</name></service>
    </servicelist>
</servicectrl>
"""

SHARELIST_NAME = "sharelist"
SHARE_PATH = "sharelist/share"
SHARE_NAME = "share"
SHARE_NAME_PATH = "sharelist/share/name"
NAME_NAME = "name"
PATH_NAME = "path"
UNDELETABLE_ATTR = "undeletable"
USB_PATH_PREFIX = "/mnt/usb"
DELETE_SHARE_NAME_PATH = "shares/share/name"
VOLUME_NAME_PATH = "volume/name"

DIXIMDMS_NAME = "diximdms"
LICENSE_KEY_PATH = "dtcp/license/key"


class EvhDiximDmsService(evhcmd.EvhCmd):
    """Event handler to start or stop services for diximdms service."""

    def __send_request(self, target, type, body=""):
        """Send request entry to "target"."""

        entry = request.Entry(type, body)
        return self.send_entry(entry, target)

    def __service_control(self, name, type):
        """Control service."""

        self.__send_request(FAL_SERVICECTRL_PATH, type,
                        SERVICECTRL_REQ_XML_TMPL % (name,))

    def __update_share_symlinks(self, delete_list=[]):
        """Update Symbolic link files for shares."""

        shares_config = self.__send_request(FAL_SHARES_PATH,
                                            ReqTypes.LOAD).get()
        diximdms_config = self.__send_request(FAL_DIXIMDMS_PATH,
                                            ReqTypes.LOAD).get()
        exported_share_list = diximdms_config.get_value_list(SHARE_NAME_PATH)
        for share in shares_config.get_node_list(SHARE_PATH):
            name = share.get_value(NAME_NAME)
            path = share.get_value(PATH_NAME)
            if not name or not path:
                continue

            if (share.is_set_to_true(UNDELETABLE_ATTR) and
                not share.get_value(PATH_NAME).startswith(USB_PATH_PREFIX)):

                # Ignore undeletable share except USB volume.
                continue

            exports_link = os.path.join(DMS_DTCP_EXPORTS_PATH, name)
            if (name in exported_share_list and
                os.path.exists(path) and
                name not in delete_list):

                if not os.path.exists(exports_link): 
                    os.symlink(path, exports_link)
            else:
                if os.path.islink(exports_link): 
                    os.remove(exports_link)

    def __prepare_for_service(self):
        """Prepare for diximdms service."""

        if not os.path.exists(DMS_DATABASE_PATH):
            os.makedirs(DMS_DATABASE_PATH)

        database_src_path = DMS_DATABASE_SRC_PATH
        if self.__is_dms_dtcp_service_enabled():
            database_src_path = DMS_DTCP_DATABASE_SRC_PATH
        for src_path in glob.glob(os.path.join(database_src_path, "*")):
            dest_path = os.path.join(DMS_DATABASE_PATH,
                                        os.path.basename(src_path))
            if os.path.exists(dest_path):
                if os.stat(src_path).st_mtime <= os.stat(dest_path).st_mtime:
                    continue

            shutil.copy2(src_path, dest_path)

        for prepare_dir in DMS_DTCP_PREPARE_DIR_LIST:
            if not os.path.exists(prepare_dir):
                os.makedirs(prepare_dir)

        self.__update_share_symlinks()

    def __is_dms_dtcp_service_enabled(self):
        """Check if DMS service supporting DTCP-IP is enabled."""

        try:
            diximdms_query = self.__send_request(FAL_DIXIMDMS_PATH,
                                        ReqTypes.QUERY, DIXIMDMS_NAME).get()
            if diximdms_query.get_node(LICENSE_KEY_PATH).get_text():
                return True
            return False
        except:
            return False

    def __is_dms_service_enabled(self):
        """Check if "name" service is enabled."""

        try:
            if self.__is_dms_dtcp_service_enabled():
                return True

            diximdms_config = self.__send_request(FAL_DIXIMDMS_PATH,
                                            ReqTypes.LOAD).get()

            service = diximdms_config.get_node(DIXIMDMS_SERVICE_PATH)
            return service.is_enabled()
        except:
            return False

    def __handle_external_volume_event(self, eventinfo, entry, action,
        is_previous):
        """Handle external volume event."""

        try:
            vol_name = eventinfo.get_value(VOLUME_NAME_PATH)
            if os.path.exists(os.path.join(DMS_DTCP_EXPORTS_PATH, vol_name)):
                if is_previous == True:
                    self.__service_control(DMS_SERVICE, ReqTypes.SUSPEND)
                else:
                    self.__service_control(DMS_SERVICE, ReqTypes.RESUME)
        except:
            # Ignore error
            pass
            
        entry.set_result(Results.OK)
        return entry

    def __handle_volume_event(self, eventinfo, entry):
        """Handle volume event."""

        action = eventinfo.get_attr_value(ACTION_ATTR)
        is_previous = eventinfo.is_set_to_true(PREVIOUS_ATTR)

        if (eventinfo.get_node(VOLUME_NAME_PATH).get_text() !=
                config.INTERNAL_VOL):

            # Non target events
            return self.__handle_external_volume_event(eventinfo, entry,
                                                        action, is_previous)

        if action == MOUNT_ACTION:
            if not is_previous:
                try:
                    self.__prepare_for_service()
                except:
                    self.exception("Preparing fail!!")

                if self.__is_dms_service_enabled():
                    self.__service_control(DMS_SERVICE, ReqTypes.START)

        elif action == UNMOUNT_ACTION:
            if is_previous:
                self.__service_control(DMS_SERVICE, ReqTypes.STOP)

        entry.set_result(Results.OK)
        return entry

    def __handle_shares_event(self, eventinfo, entry):
        """Handle shares event."""

        action = eventinfo.get_attr_value(ACTION_ATTR)
        is_previous = eventinfo.is_set_to_true(PREVIOUS_ATTR)

        if action == DELETE_ACTION:
            if is_previous:
                delete_share = eventinfo.get_value(DELETE_SHARE_NAME_PATH)
                diximdms_config = self.__send_request(FAL_DIXIMDMS_PATH,
                                            ReqTypes.LOAD).get()
                sharelist = \
                        diximdms_config.get_node(SHARELIST_NAME).get_clone()
                idx = 0
                for share in sharelist.get_node_list(SHARE_NAME):
                    if share.get_value(NAME_NAME) == delete_share:
                        sharelist.delete_child(SHARE_NAME, index=idx)
                        break
                    idx += 1

                diximdms = XmlConfNode(DIXIMDMS_NAME)
                diximdms.append_child(sharelist)
                self.__send_request(FAL_DIXIMDMS_PATH,
                                    ReqTypes.APPLY, diximdms)

        entry.set_result(Results.OK)
        return entry

    def __handle_system_event(self, eventinfo, entry):
        """Handle system event."""

        action = eventinfo.get_attr_value(ACTION_ATTR)
        is_previous = eventinfo.is_set_to_true(PREVIOUS_ATTR)

        if action == SHUTDOWN_ACTION or action == REBOOT_ACTION:
            if is_previous:
                self.__service_control(DMS_SERVICE, ReqTypes.STOP)

        entry.set_result(Results.OK)
        return entry

    def __handle_diximdms_event(self, eventinfo, entry):
        """Handle diximdms event."""

        action = eventinfo.get_attr_value(ACTION_ATTR)
        is_previous = eventinfo.is_set_to_true(PREVIOUS_ATTR)

        if action == APPLY_ACTION:
            if not is_previous:
                self.__update_share_symlinks();

        entry.set_result(Results.OK)
        return entry

    def __event_handler(self, entry):
        """Handle "event" request."""

        try:
            eventinfo = entry.get().get_node(EVENTINFO_NAME)
            if eventinfo.get_attr_value(CATEGORY_ATTR) == VOLUME_CATEGORY:
                return self.__handle_volume_event(eventinfo, entry)
            elif eventinfo.get_attr_value(CATEGORY_ATTR) == SHARES_CATEGORY:
                return self.__handle_shares_event(eventinfo, entry)
            elif eventinfo.get_attr_value(CATEGORY_ATTR) == SYSTEM_CATEGORY:
                return self.__handle_system_event(eventinfo, entry)
            elif eventinfo.get_attr_value(CATEGORY_ATTR) == DIXIMDMS_CATEGORY:
                return self.__handle_diximdms_event(eventinfo, entry)

        except:
            # Non target events
            pass

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)


#
# Main
#

if __name__ == "__main__":
    EvhDiximDmsService(option=evhcmd.EvhCmdOption()).handle_request()
