/**
 * @module dixim
 * @submodule contents
 */
dixim.namespace('dixim.contents');

/**
 * @module contents
 * @submodule rename-dialog-controller
 */
(function() {
var RenameDialogController;
var prototype;
var superConstructor;
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var RenameDialogView = dixim.contents.RenameDialogView;
var ApiClientTransfer = dixim.common.ApiClientTransfer;
var _ = dixim.ctt.ResourceBundle.get;
var Configs = dixim.ctt.Configs;
var ResourceBundle = dixim.ctt.ResourceBundle;

/**
 * @class RenameDialogController
 * @constructor
 * @param {Object} configs
 * @param {MobileTransferApiClient} [configs.model]
 */
RenameDialogController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(RenameDialogController, EventTarget);
dixim.contents.RenameDialogController = RenameDialogController;
prototype = RenameDialogController.prototype;
superConstructor = RenameDialogController.superConstructor;

var UPDATE = 'update';
var CLOSE = 'close';
var ErrorCode =  {
    EMPTY_STRING: 'empty',
    EMOJI_STRING: 'emoji',
    FORBIDDEN: 406,
    CONFLICTED: 409,
    EXCEEDS_MAX_LENGTH: 413,
    IN_PROGRESS: 513
};
var ErrorMessage = (function() {
    var ret = {};
    var i = null;
    var code;
    var text;

    ResourceBundle.reset();
    Configs.set('resourceBundlePath', 'lang/contents/');

    for (i in ErrorCode) {
        if (!ErrorCode.hasOwnProperty(i)) continue;
        code = ErrorCode[i];
        text = _(['contents_rename_dialog_error_', code].join(''));
        ret[code] = text;
    }
    return ret;
}());

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 * @param {Object} configs.targetItem
 * @param {MobileTransferApiClient} [configs.model]
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    this._model = configs.model || new ApiClientTransfer();
    this._model.addEventListener('rename', this._successRename, this);
    this._model.addEventListener('renameError', this._errorRename, this);

    this._view = new RenameDialogView(configs);
    this._view.addEventListener('render', this._clickRenderView, this);
    this._view.addEventListener('rename', this._clickRename, this);
    this._view.addEventListener('cancel', this._clickCancel, this);

    this._targetItem = configs.targetItem;
    this._session = null;
};

/**
 * @method start
 */
prototype.start = function() {
    this._view.show();
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype._clickRenderView = function(e) {
    this._view.updateTargetItem(this._targetItem);
};

prototype._validateName = function(name) {
    var getStringSize = function(str) {
        // Matches only the 10.. bytes that are non-initial characters in a multi-byte sequence.
        var ret = encodeURIComponent(str).match(/%[89ABab]/g);

        return str.length + (ret ? ret.length : 0);
    };
    var isEmoji = function(str) {
        var strCode;
        var el = [
            0xa9, 0xae, 0x20e3, 0x2122, 0x2139, 0x23f0, 0x23f3, 0x24c2, 0x25b6, 0x25c0, 0x260e, 0x2611, 0x261d, 0x263a, 0x2660,
            0x2663, 0x2668, 0x267b, 0x267f, 0x2693, 0x26d4, 0x26ce, 0x26ea, 0x26f5, 0x26fa, 0x26fd, 0x2702, 0x2705, 0x270f, 0x2714,
            0x2716, 0x2728, 0x274c, 0x274e, 0x2757, 0x27a1, 0x27bf, 0x2712, 0x2744, 0x2747, 0x2764, 0x27b0, 0x2b50, 0x2b55, 0x3030,
            0x303d, 0x3297, 0x3299,
        ];
        var el_range = [
            {start: 0xd83c, end: 0xd83d},
            {start: 0x2194, end: 0x2199},
            {start: 0x21a9, end: 0x21aa},
            {start: 0x231a, end: 0x231b},
            {start: 0x23e9, end: 0x23ec},
            {start: 0x25aa, end: 0x25ab},
            {start: 0x25fb, end: 0x25fe},
            {start: 0x2600, end: 0x2601},
            {start: 0x2614, end: 0x2615},
            {start: 0x2648, end: 0x2653},
            {start: 0x2665, end: 0x2666},
            {start: 0x26a0, end: 0x26a1},
            {start: 0x26aa, end: 0x26ab},
            {start: 0x26c4, end: 0x26c5},
            {start: 0x26bd, end: 0x26be},
            {start: 0x26f2, end: 0x26f3},
            {start: 0x2708, end: 0x2709},
            {start: 0x270a, end: 0x270c},
            {start: 0x2733, end: 0x2734},
            {start: 0x2753, end: 0x2755},
            {start: 0x2795, end: 0x2797},
            {start: 0x2934, end: 0x2935},
            {start: 0x2b05, end: 0x2b07},
            {start: 0x2b1b, end: 0x2b1c},
            {start: 0xe63e, end: 0xe757},
            {start: 0xe7d0, end: 0xe83b},
            {start: 0xe960, end: 0xe988},
            {start: 0xeaf0, end: 0xeb29},
            {start: 0xeb2a, end: 0xeba3},
            {start: 0xee10, end: 0xee33},
            {start: 0xee40, end: 0xee4a},
            {start: 0xee70, end: 0xee7d},
        ];

        for (var i = 0; i < str.length; i += 1) {
            strCode = str.charCodeAt(i);
            if (strCode >= parseInt(0xd83c, 10) && strCode <= parseInt(0xd83d)) return true;

            for (var j = 0; j < el.length; j += 1) {
                if (strCode == parseInt(el[j], 10)) return true; //emoji
            }
            for (var k = 0; k < el_range.length; k += 1) {
                if (strCode >= parseInt(el_range[k].start, 10) && strCode <= parseInt(el_range[k].end)) return true; // emoji
            }
        }

        return false;
    };

    if (!!!name) {
        return ErrorCode.EMPTY_STRING;
    } else if (getStringSize(name) > 255) {
        return ErrorCode.EXCEEDS_MAX_LENGTH;
    } else if (isEmoji(name)) {
        return ErrorCode.EMOJI_STRING;
    }

    return null;
};

prototype._clickRename = function(e) {
    var args = e.args;
    var newName = args.newName;
    var item = args.targetItem;
    var error;

    if (!!this._session) return;

    if (!!(error = this._validateName(newName))) {
        this._view.showError(ErrorMessage[error]);
        return;
    }
    this._view.showActivityIndicator();
    this._session = this._model.rename({id: item.refId ? item.refId : item.id, new_name: newName});
};

prototype._successRename = function(e) {
    this.stop();
    this.dispatchEvent(new Event(UPDATE));
    this._session = null;
};

prototype._errorRename = function(e) {
    var code = e.args.errorCode;

    this._view.hideActivityIndicator();
    this._view.showError(ErrorMessage[code] || _('contents_rename_dialog_error_unknown'));
    this._session = null;
};

prototype._clickCancel = function(e) {
    if (!!this._session) return;

    this.stop();
    this.dispatchEvent(new Event(CLOSE));
};

}());
