/**
 * @module dixim
 * @submodule download
 */
dixim.namespace('dixim.download');

/**
 * @module download 
 * @submodule download-view
 */

(function() {
var DownloadView;
var prototype;
var superConstructor;
var Formatter = dixim.ctt.util.Formatter;
var _ = dixim.ctt.ResourceBundle.get;
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var Configs = dixim.ctt.Configs;
var UiUtils = dixim.common.UiUtils;
var $ = dixim.ctt.jQuery;
var Template = dixim.ctt.util.Template;
var escape = dixim.xml.escape;

/**
 * @class DownloadView
 * @constructor
 * @param {Object} configs
 */
DownloadView = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(DownloadView, EventTarget);
dixim.contents.DownloadView = DownloadView;
prototype = DownloadView.prototype;
superConstructor = DownloadView.superConstructor;

var SELECT = 'select';
var DOWNLOAD = 'download';
var SELECT_SERVER = 'selectServer';
var SELECT_CONTENT = 'selectContent';
var REFRESH = 'refresh';
var SETTING = 'setting';
var PREV = 'prev';

var TEMPLATE_LOADING = Template.LOADING_ICON_LARGE;
var CONTENTS_IMAGES = {
    '-1': [Configs.get('imagePath'), '/ico_list_mvno.png'].join(''),
    '1': [Configs.get('imagePath'), '/ico_list_mv1.png'].join(''),
    '2': [Configs.get('imagePath'), '/ico_list_mv2.png'].join(''),
    '3': [Configs.get('imagePath'), '/ico_list_mv3.png'].join(''),
    '4': [Configs.get('imagePath'), '/ico_list_mv4.png'].join(''),
    '5': [Configs.get('imagePath'), '/ico_list_mv5.png'].join(''),
    '6': [Configs.get('imagePath'), '/ico_list_mv6.png'].join(''),
    '7': [Configs.get('imagePath'), '/ico_list_mv7.png'].join(''),
    '8': [Configs.get('imagePath'), '/ico_list_mv8.png'].join(''),
    '9': [Configs.get('imagePath'), '/ico_list_mv9.png'].join(''),
    '10': [Configs.get('imagePath'), '/ico_list_mv10.png'].join(''),
    'uploadable': [Configs.get('imagePath'), '/ico_list_mvno.png'].join(''),
    'free': [Configs.get('imagePath'), '/ico_list_mv.png'].join(''),
    'nosecureV': [Configs.get('imagePath'), '/ico_list_nosecure_v.png'].join(''),
    'nosecureM': [Configs.get('imagePath'), '/ico_list_nosecure_m.png'].join(''),
    'nosecureP': [Configs.get('imagePath'), '/ico_list_nosecure_p.png'].join(''),
    'folder': [Configs.get('imagePath'), '/ico_list_folder.png'].join(''),
};

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    this._title = _('download_menu_download');
    this._browsedData = {};
    this._contentName = new this._contentNameClosure();
};

/**
 * @method show
 */
prototype.show = function() {
    $('.rootPage').html([
        '<div class="contents" data-enhance="false">',
        '<h2 id="tit_page">', this._title, '</h2>',
        '<div class="contents_inner">',
        '<ul id="contentField" class="content_field list_btn list_next list_menu">',
        '</ul>',
        '</div>',
        '</div>',
    ].join(''));

    UiUtils.cleanActionSettingBar();
    UiUtils.cleanActionBar();
    UiUtils.setActionBar('ab_select', dixim.proxy(this._clickSelect, this));
    UiUtils.setActionBar('ab_download', dixim.proxy(this._clickDownload, this));
    UiUtils.disableActionBar('ab_select');
    UiUtils.disableActionBar('ab_download');
    UiUtils.showActionSettingBar('ac_refresh', dixim.proxy(this._clickRefresh, this));
    UiUtils.showActionSettingBar('ac_setting', dixim.proxy(this._clickSetting, this));

    this.setContentName(this._title);
    $('.btn_prev').off('click').show().on('click', dixim.proxy(this._clickPrev, this));
};

/**
 * @method hide
 */
prototype.hide = function() {
   $('.rootPage').empty();
};

prototype._contentNameClosure = function() {
    var stack = [];

    return {
        push: function(name) {
            $('.art_tit').text(name);
            stack.push(name);
        },
        pop: function() {
            stack.pop();
            $('.art_tit').text(stack[stack.length - 1]);
        },
    }
};

prototype.startLoading = function() {
    this.stopLoading();
    $('.contents_inner').append([
        '<div id="loading_icon">', TEMPLATE_LOADING, '</div>',
    ].join(''));
};

prototype.stopLoading = function() {
    $('#loading_icon').remove();
};

prototype.setContentName = function(name) {
    this._contentName.push(name);
};

prototype.unsetContentName = function() {
    this._contentName.pop();
};

prototype.addRow = function(data) {
    var img = data.deviceIcon;
    var rowSelector;

    $('#contentField').append([
        '<li>',
        '<table>',
        '<tr>',
        '<td class="server-img-field">', '<img class="contents-img" src="', img, '">', '</td>',
        '<td>', '<p class="content-title">', escape(data.friendlyName), '</p>', '</td>',
        '</tr>',
        '</table>',
        '</li>',
    ].join(''));

    rowSelector = $('#contentField li:last-child');

    rowSelector.on('click', data, dixim.proxy(this._clickSelectServer, this));
};

prototype.noRow = function(isServer) {
    if (isServer) {
        this._error(_('download_no_server_error'));
    } else {
        this._error(_('download_no_contents_error'));
    }
};

prototype.errorRow = function(isServer) {
    if (isServer) {
        this._error(_('download_fetch_server_error'));
    } else {
        this._error(_('download_fetch_contents_error'));
    }
};

prototype._error = function(text) {
    $('#contentField').html([
        '<div class="txt_none error-text-field">',
        '<span>', text, '</span>',
        '</div>',
    ].join(''));
};

prototype._getDefaultImage = function(data) {
    switch(data.mediaType) {
        case 'video':
            if (/^DTCP_/.test(data.profile) || (data.profile === '' && /.dtcp$/.test(data.url))) {
                if (typeof data.copyCount === 'undefined') {
                    return (data.uploadable) ? CONTENTS_IMAGES['free'] : CONTENTS_IMAGES['uploadable'];
                } else {
                    return CONTENTS_IMAGES[data.copyCount];
                }
            }
            return CONTENTS_IMAGES['nosecureV'];
        case 'audio':
            return CONTENTS_IMAGES['nosecureM'];
        case 'image':
            return CONTENTS_IMAGES['nosecureP'];
    }
};

prototype._getImageSrc = function(data) {
    if (data.class === 'container') {
        return CONTENTS_IMAGES['folder'];
    }

    if (typeof data.thumbnailUrl !== 'undefined' && data.thumbnailUrl !== '') return data.thumbnailUrl;

    return this._getDefaultImage(data);
};

prototype.addContent = function(data) {
    var id = $('#contentField li').length;
    var rowSelector;

    this._browsedData[id] = data;

    $('#contentField').append([
        '<li>',
        '<table>',
        '<tr>',
        '<td id="checkbox_', id, '" class="contents_icon">', '</td>',
        '<td class="contents_icon">', '<img class="contents-img" src="', this._getImageSrc(data), '">', '</td>',
        '<td class="contents_title">',
        '<p class="content-title txt_m">', escape(data.title), '</p>',
        '<div class="list_detail txt_s">', '</div>',
        '</td>',
        '</tr>',
        '</table>',
        '</li>',
    ].join(''));

    rowSelector = $('#contentField li:last-child');

    rowSelector.find('img').error(dixim.proxy(function() {
        rowSelector.find('img').prop('src', this._getDefaultImage(data));
    }, this));

    if (data.class === 'container') {
        rowSelector.addClass('content_folder');
        rowSelector.on('click', data, dixim.proxy(this._clickSelectContent, this));
    } else {
        rowSelector.addClass('list_content');
        rowSelector.find(['#checkbox_', id].join('')).html([
            '<label class="checkbox_contents">', '<input type="checkbox" id="check_', id, '"/>',
            '<span>', '</span>',
            '</label>',
        ].join(''));
        if (typeof data.size !== 'undefined') {
            rowSelector.find('.list_detail').append([
                '<p class="content-info">', Formatter.stringifySize(data.size), '</p>',
            ].join(''));
        }
        if (typeof data.date !== 'undefined') {
            rowSelector.find('.list_detail').append([
                '<p class="content-info">', Formatter.stringifyDate(Formatter.dcDateToDate(data.date)), '</p>',
            ].join(''));
        }
        if (!data.dn_movable) {
            rowSelector.addClass('list_nodl');
            rowSelector.find(['#checkbox_', id].join('')).remove();
        }
    }
};

prototype.checkActionBarStatus = function() {
    if ($('#contentField [type="checkbox"]').length < 1) {
        UiUtils.disableActionBar('ab_select');
        UiUtils.disableActionBar('ab_download');
    } else {
        UiUtils.enableActionBar('ab_select');
        UiUtils.enableActionBar('ab_download');
    }
};

prototype.switchSelect = function() {
    var checkboxSelector = $('#contentField [type="checkbox"]');
    var numberOfNotChecked = checkboxSelector.not(':checked').length;

    checkboxSelector.prop('checked', (numberOfNotChecked > 0));
};

prototype.cleanContent = function() {
    this._browsedData = {};

    $('#contentField').empty();
};

prototype._clickRefresh = function() {
    this.dispatchEvent(new Event(REFRESH));
};

prototype._clickSetting = function() {
    this.dispatchEvent(new Event(SETTING));
};

prototype._clickSelect = function() {
    this.dispatchEvent(new Event(SELECT));
};

prototype._clickDownload = function() {
    var browsedData = this._browsedData;
    var dataList = [];

    $('#contentField [type="checkbox"]:checked').each(function() {
        dataList.push(browsedData[$(this).prop('id').substr(6)]);
    });
    this.dispatchEvent(new Event(DOWNLOAD, {dataList: dataList}));
};

prototype._clickSelectContent = function(e) {
    this.dispatchEvent(new Event(SELECT_CONTENT, e.data));
};

prototype._clickSelectServer = function(e) {
    this.dispatchEvent(new Event(SELECT_SERVER, e.data));
};

prototype._clickPrev = function() {
    this.dispatchEvent(new Event(PREV));
};

}());
