/**
 * @module dixim
 * @submodule server
 */
dixim.namespace('dixim.server');

/**
 * @module server
 * @submodule public-folder-controller
 */

(function() {
var PublicFolderController;
var prototype;
var superConstructor;
var PublicFolderView = dixim.server.PublicFolderView;
var EventTarget = dixim.event.EventTarget;
var _ = dixim.ctt.ResourceBundle.get;
var ApiClientServer = dixim.common.ApiClientServer;
var ErrorModalDialogView = dixim.common.ErrorModalDialogView;
var Enum = dixim.ctt.util.Enum;
var PublicFolderWizardDialogController = dixim.server.PublicFolderWizardDialogController;
var MediaTypeDialogController = dixim.server.MediaTypeDialogController;
var DeletionDialogController = dixim.server.DeletionDialogController;
var ProgressDialogController = dixim.server.ProgressDialogController;
var Template = dixim.ctt.util.Template;

/**
 * @class PublicFolderController
 * @constructor
 * @param {Object} configs
 */
PublicFolderController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(PublicFolderController, EventTarget);
dixim.server.PublicFolderController = PublicFolderController;
prototype = PublicFolderController.prototype;
superConstructor = PublicFolderController.superConstructor;

var SELECT = 'select';
var DELETE = 'delete';
var CHANGE = 'change';
var ADD = 'add';
var SYNC = 'sync';
var PREV = 'prev';
var UPDATE = 'update';

var Mode = Enum.fetch('MEDIA_TYPE');
var DeletionMode = Enum.fetch('DELETION');
var ProgressMode = Enum.fetch('PROGRESS');
var SEPARATE = Template.SEPARATE;

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    configs.id = configs.id || '';
    this._id = [configs.id, 'public-folder'].join('');

    this._model = new ApiClientServer();
    this._model.addEventListener('browseFileSystem', this._successBrowseFileSystem, this);
    this._model.addEventListener('browseFileSystemError', this._errorBrowseFileSystem, this);
    this._model.addEventListener('fetchSharedFolder2', this._successFetchSharedFolder, this);
    this._model.addEventListener('fetchSharedFolder2Error', this._errorFetchSharedFolder, this);

    this._view = new PublicFolderView(configs);
    this._view.addEventListener(SELECT, this._clickSelect, this);
    this._view.addEventListener(DELETE, this._clickDelete, this);
    this._view.addEventListener(CHANGE, this._clickChange, this);
    this._view.addEventListener(ADD, this._clickAdd, this);
    this._view.addEventListener(SYNC, this._clickSync, this);
    this._view.addEventListener(PREV, this._clickPrev, this);

    this._parentContext = configs.parentContext;
    this._rootPathList;
};

/**
 * @method start
 */
prototype.start = function() {
    this._view.show();
    this._model.browseFileSystem({path: '', enableHiddenDir: 0});
    this._model.fetchSharedFolder2();
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype._successBrowseFileSystem = function(e) {
    this._rootPathList = e.args.dirs;
};

prototype._errorBrowseFileSystem = function() {
    ;
};

prototype._successFetchSharedFolder = function(e) {
    var sharedFolders;
    var sharedFoldersLen;
    var len;
    var path;
    var rootPath;
    var pos;
    var startPos;

    if (typeof this._rootPathList === 'undefined') {
        setTimeout(dixim.proxy(function() {
            this._successFetchSharedFolder(e);
        }, this), 100);
        return;
    }

    sharedFolders = e.args.sharedFolders;
    sharedFoldersLen = sharedFolders.length;
    if (sharedFoldersLen === 0) {
        this._view.noRow();
        return;
    }

    if (sharedFoldersLen > 10000) sharedFoldersLen = 10000;

    len = this._rootPathList.length;
    for (var i = 0; i < sharedFoldersLen; i += 1) {
        sharedFolders[i].id = i;
        path = sharedFolders[i].path;

	if (path == "/mnt/sataraid1/opt/dms-dtcp/exports/recorded/") continue;

        for (var j = 0; j < len; j += 1) {
            if (len <= 1) {
                if (path.indexOf(this._rootPathList[j]) !== -1) {
                    path = path.replace(this._rootPathList[j], '');
                    break;
                }
            } else {
                rootPath = this._rootPathList[j];
                startPos = rootPath.length - 2;
                pos = rootPath.lastIndexOf(SEPARATE, startPos) + 1;
                rootPath = rootPath.substring(0, pos);
                if (this._rootPathList[j] === SEPARATE) continue;
                if (path.indexOf(this._rootPathList[j]) !== -1) {
                    path = path.replace(rootPath, '');
                    break;
                }
            }
        }
        this._view.addRow(sharedFolders[i], path);
    }
};

prototype._errorFetchSharedFolder = function() {
    this._errorDialog(_('server_public_folder_fetch_error'));
};

prototype._clickSelect = function() {
    this._view.switchSelect();
};

prototype._clickDelete = function(e) {
    var query = {};
    var controller;
    var sharedFolders = e.args.sharedFolders;
    var sharedFoldersLen = sharedFolders.length;

    if (sharedFoldersLen === 0) {
        this._errorDialog(_('server_public_folder_error_no_select'));
        return;
    }
    query.contents = sharedFolders;
    query.mode = DeletionMode.SHARED_FOLDER;
    controller = new DeletionDialogController(query);
    controller.addEventListener(UPDATE, this._update, this);
    controller.start();
};

prototype._clickChange = function(e) {
    var query = {};
    var controller;
    var sharedFolders = e.args.sharedFolders;
    var sharedFoldersLen = sharedFolders.length;

    if (sharedFoldersLen !== 1) {
        this._errorDialog(_('server_public_folder_error_one_select'));
        return;
    }
    query.sharedFolder = sharedFolders[0];
    controller = new MediaTypeDialogController(query);
    controller.addEventListener(UPDATE, this._update, this);
    controller.start();
};

prototype._clickAdd = function() {
    var query = {};
    var controller;

    controller = new PublicFolderWizardDialogController(query);
    controller.addEventListener(UPDATE, this._update, this);
    controller.start();
};

prototype._clickSync = function(e) {
    var query = {};
    var controller;
    var sharedFolders = e.args.sharedFolders;
    var sharedFoldersLen = sharedFolders.length;

    if (sharedFoldersLen === 0) {
        this._errorDialog(_('server_public_folder_error_no_select'));
        return;
    }
    query.contents = sharedFolders;
    query.mode = ProgressMode.SYNC_PUBLIC;
    controller = new ProgressDialogController(query);
    controller.start();
};

prototype._clickPrev = function() {
    this.stop();
    this._parentContext.start();
};

prototype._update = function() {
    this._view.cleanList();
    this._model.fetchSharedFolder2();
};

prototype._errorDialog = function(text) {
    var dialog;
    var query = {};

    query.id = this._id;
    query.title = _('server_menu_public_folder');
    query.description = text;
    dialog = new ErrorModalDialogView(query);
    dialog.show();
};

}());
