/**
 * @module ctt
 * @submodule item-listing-view
 */
(function() {
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var $ = dixim.ctt.jQuery;
var encodeId = dixim.xml.encodeId;
var decodeId = dixim.xml.decodeId;
var escapeHtml = dixim.xml.escape;

var ItemListingView;
var prototype;

/**
 * ItemListingView represents a item list which has multiple rows.
 * Each row has two columns, the icon column and the title column.
 *
 * @class ItemListingView
 * @extends EventTarget
 * @constructor
 * @param {Object} configs
 * @param {String} configs.containerNode
 */
ItemListingView = function(configs) {
  EventTarget.call(this);
  this._initialize(configs);
};
dixim.inherit(ItemListingView, EventTarget);
dixim.ctt.ItemListingView = ItemListingView;
prototype = ItemListingView.prototype;

/**
 * The render event is sent when
 * the container has been rendered.
 * 
 * @event render
 */
var RENDER = 'render';

/**
 * Triggered when an item of the list has been clicked.
 *
 * @event clickItem
 * @param {Event} event
 * @param {Object} event.args
 * @param {Object} event.args.record
 */
var CLICK_ITEM = 'clickItem';

/**
 * Triggered when the list selection has been changed.
 */
var CHANGE_SELECTION = 'changeSelection';

/**
 * Template of list container.
 *
 * @property TEMPLATE_CONTAINER
 * @static
 * @private
 */
var TEMPLATE_CONTAINER = '<ul class="ctt-item-listing-view"/>';

/**
 * Template of a row of the list.
 *
 * There are the following placeholders that are replaced by
 * the corresponded property of the record object.
 *
 * * ${id} replaced by record.id, but prefixed by ELEMENT_ID_PREFIX and encoded accordingly.
 * * ${title} replaced by record.title.
 * * ${iconContainer} see <em>TEMPLATE_ROW_ICON</em>.
 *
 * @property TEMPLATE_ROW
 * @static
 * @private
 */
var TEMPLATE_ROW = ['<li id="${id}" class="ctt-item-listing-view-row">',
                     '${iconContainer}',
                     '<div class="ctt-item-listing-view-row-title">${title}</div>',
                     '</li>'].join('');

/**
 * Template of an icon element of the row.
 * 
 * The "${iconContainer}" placeholder of the <em>TEMPLATE_ROW</em> is
 * replaced by this element. 
 *
 * Placeholders:
 *
 * * ${icon} replaced by record.icon
 *
 * @property TEMPLATE_ROW_ICON
 * @static
 * @private
 */
var TEMPLATE_ROW_ICON = ['<div class="ctt-item-listing-view-row-thumbnail-wrapper">',
                          '<img class="ctt-item-listing-view-row-thumbnail" src="${icon}"/>',
                          '</div>'].join('');

/**
 * @property ELEMENT_ID_PREFIX
 * @static
 * @private
 */
var ELEMENT_ID_PREFIX = 'ctt-item-listing-view-';

/**
 * TODO: this is not responsible for the View.
 * Each view should only define required fields.
 *
 * Default schema.
 *
 * By default, the schema is defined as follows.
 *
 *     var SCHEMA = {
 *       id: 'id',
 *       title: 'title',
 *       icon: 'icon'
 *     };
 *
 * @property SCHEMA
 * @static
 * @private
 */
var SCHEMA = {
  id: 'id',
  title: 'title',
  icon: 'icon'
};

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 * @param {String} configs.containerNode
 * @param {String} [configs.elementIdPrefix=ELEMENT_ID_PREFIX]
 * @param {String} [configs.className] Element class name prepended to the class of the list container element.
 * @param {Object} [configs.schema=SCHEMA] an object that specifies which property of the record corresponds to the column.
 * @param {String} [configs.selectionMode="single"] "single" | "multiple".
 */
prototype._initialize = function(configs) {
  var schema;

  if (!!!(this._containerNode = configs.containerNode)) {
    throw new Error('containerNode is required');
  }
  this._className = configs.className;
  this._elementIdPrefix = configs.elementIdPrefix || ELEMENT_ID_PREFIX;
  if (configs.schema) {
    schema = dixim.extend({}, SCHEMA, true);
    this._schema = dixim.extend(schema, configs.schema, true);
  }
  this._containerTemplate = configs.containerTemplate || TEMPLATE_CONTAINER;
  this._rowTemplate = configs.rowTemplate || TEMPLATE_ROW;
  this._rowIconTemplate = configs.rowIconTemplate || TEMPLATE_ROW_ICON;

  this._selectionMode = configs.selectionMode || 'single';
  this._selection = {};
  this._records = [];
};

/**
 * @method _createView
 * @protected
 */
prototype._createView = function() {
  var v;
  var cls;

  v = $(this._containerTemplate).hide().appendTo(this._containerNode);
  if (!!this._className) {
    cls = v.attr('class');
    v.attr({'class': [this._className, cls].join(' ')});
  }
  v.on('click', '.ctt-item-listing-view-row', $.proxy(this._didClickItem, this));
  this._view = v;
};

prototype._didClickItem = function(e) {
  var row = $(e.currentTarget);
  var re = new RegExp('^' + this._elementIdPrefix);
  var args = {};
  var id;

  id = row.attr('id');
  id = decodeId(id.replace(re, ''));

  if (!!!(args.record = this._getRecord(id))) return;

  this.dispatchEvent(new Event(CLICK_ITEM, args));
};

/**
 * Returns a record that is managed by this list.
 *
 * @method _getRecord
 * @param {Object|String} row record object or record id to get.
 * @return {Object} a record object found, null otherwise.
 */
prototype._getRecord = function(row) {
  var records = this._records;
  var len = records.length;
  var targetId = null;
  var i;
  var rid;

  if (typeof row === 'object') {
    targetId = this._getRecordProperty(row, 'id');
  } else if (typeof row === 'string') {
    targetId = row;
  }
  if (!!!targetId) return null;

  for (i = 0; i < len; i += 1) {
    rid = this._getRecordProperty(records[i], 'id');
    if (rid === targetId) return records[i];
  }
  return null;
};

/**
 * @method show
 */
prototype.show = function() {
  if (!!!this._view) {
    this._createView();
  }
  this._view.show();

  this.dispatchEvent(new Event(RENDER));
};

/**
 * @method hide
 */
prototype.hide = function() {
  this._view.hide();
};

/**
 * @method destroy
 */
prototype.destroy = function() {
  this._view.hide();
  this._view.remove();
  this._view = null;
};

/**
 * Adds rows for data records specified.
 *
 * Each record must have properties which are specified
 * in <em>SCHEMA</em>, or mapped to the properties in the <em>SCHEMA</em>.
 *
 * @method addRows
 * @param {Object[]} records
 */
prototype.addRows = function(records) {
  this._records = this._records.concat(records);
  this._addRowElements(records);
};

/**
 * @method _addRowElements
 * @param {Object[]} records
 * @protected
 */
prototype._addRowElements = function(records) {
  var len = records.length;
  var container = this._view;
  var i;
  var row;
  var record;

  for (i = 0; i < len; i += 1) {
    record = records[i];
    row = this._createRowElementHtml(record, i);
    if (!!!row) continue;
    row = this._createRowElementNode(record, i, row);
    row.appendTo(container);
  }
};

/**
 * @method _createRowElementHtml
 * @param {Object} record
 * @param {Number} index
 * @return {String} html string.
 * @protected
 */
prototype._createRowElementHtml = function(record, index) {
  var icon;
  var iconContainer;
  var id;
  var html;
  var title;

  if (!!(icon = this._getRecordProperty(record, 'icon'))) {
    iconContainer = this._rowIconTemplate.replace('${icon}', icon);
  } else {
    iconContainer = '';
  }

  if (!!!(id = this._makeElementId(record))) {
    return null;
  }

  title = this._getRecordProperty(record, 'title') || '';
  html = this._rowTemplate.
         replace('${id}', id).
         replace('${title}', escapeHtml(title)).
         replace('${iconContainer}', iconContainer);

  return html;
};

/**
 * @method _createRowElementNode
 * @param {Object} record
 * @param {Number} index
 * @param {String} html
 * @protected
 * @return {jQuery}
 */
prototype._createRowElementNode = function(record, index, html) {
  return $(html);
};

/**
 * @method _makeElementId
 * @param {Object} record
 * @protected
 */
prototype._makeElementId = function(record) {
  var id;

  if (!!!(id = this._getRecordProperty(record, 'id'))) {
    return null;
  }

  return [this._elementIdPrefix, encodeId(id)].join('');
};

/**
 * Returns a property value of the specified record object
 * which is mapped to the column specified.
 *
 * @method _getRecordProperty
 * @param {Object} record
 * @param {String} name column name.
 * @protected
 */
prototype._getRecordProperty = function(record, name) {
  var key;

  if (this._schema) {
    key = this._schema[name];
  } else {
    key = name;
  }
  return record[key];
};

/**
 * @method selectRow
 * @param {Object|String} row
 */
prototype.selectRow = function(row) {
  this._selectRow(row);
};

prototype._selectRow = function(row) {
  this._selectRowImpl(row);
  this._didChangeSelection();
};

// doesn't trigger event
prototype._selectRowImpl = function(row) {
  var record;
  var id;

  if (!!!(record = this._getRecord(row))) return;
  if (this._isSelectedRow(record)) return;

  if (this._selectionMode !== 'multiple') {
    this._clearSelectionImpl();
  }
  this._selectRowElement(record);

  if (!!!(id = this._getRecordProperty(record, 'id'))) return;
  this._selection[id] = record;
};

prototype._clearSelection = function() {
  this._clearSelectionImpl();
  this._didChangeSelection();
};

// doesn't trigger event
prototype._clearSelectionImpl = function() {
  var id = null;
  var selection = this._selection;

  for (id in selection) {
    if (!selection.hasOwnProperty(id)) continue;
    this._unselectRowImpl(selection[id]);
  }
  selection = {};
};

prototype._didChangeSelection = function() {
  this.dispatchEvent(new Event(CHANGE_SELECTION));
};

/**
 * @method getRowElement
 * @param {Object} record
 * @return {jQuery}
 */
prototype.getRowElement = function(record) {
  return this._getRowElement(record);
};

/**
 * @method _getRowElement
 * @param {Object} record
 * @return {jQuery}
 */
prototype._getRowElement = function(record) {
  var el;

  el = $(['#', this._makeElementId(record)].join(''), this._view);
  if (!el.length) return null;
  return el;
};

prototype._selectRowElement = function(record) {
  var el;

  el = this._getRowElement(record);

  if (!!el) {
    el.addClass('ctt-item-listing-view-row-selected');
  }
};

prototype._unselectRow = function(row) {
  this._unselectRowImpl(row);
  this._didChangeSelection();
};

// doesn't trigger event
prototype._unselectRowImpl = function(row) {
  var record;
  var id;

  if (!!!(record = this._getRecord(row))) return;
  if (!this._isSelectedRow(record)) return;

  this._unselectRowElement(record);
  if (!!!(id = this._getRecordProperty(record, 'id'))) return;
  delete this._selection[id];
};

prototype._unselectRowElement = function(record) {
  var el;

  el = this._getRowElement(record);
  if (!!el) {
    el.removeClass('ctt-item-listing-view-row-selected');
  }
};

prototype.getSelectedRows = function() {
  return this._getSelectedRows();
};

prototype._getSelectedRows = function() {
  var ret = [];
  var id = null;
  var selection = this._selection;

  for (id in selection) {
    if (!selection.hasOwnProperty(id)) continue;
    ret.push(selection[id]);
  }
  return ret;
};

prototype._isSelectedRow = function(row) {
  var id;

  if (!!!(id = this._getRecordProperty(row, 'id'))) {
    return false;
  }
  return !!this._selection[id];
};

/**
 * @method toggleRowSelection
 * @param {Object|String} row
 */
prototype.toggleRowSelection = function(row) {
  this._toggleRowSelection(row);
};

prototype._toggleRowSelection = function(row) {
  if (this._isSelectedRow(row)) {
    this._unselectRowImpl(row);
  } else {
    this._selectRowImpl(row);
  }
  this._didChangeSelection();
};

/**
 * @method removeAllRows
 */
prototype.removeAllRows = function() {
  this._view.empty();
  this._records = [];
  this._selection = {};
  this._didChangeSelection();
};

prototype.getNumberOfRows = function() {
  return this._records.length;
};

/**
 * @method selectRowAtIndex
 * @param {Number} index
 */
prototype.selectRowAtIndex = function(index) {
  var records = this._records;
  var len = records.length;

  if (index < len) {
    this._selectRow(records[index]);
  }
};

}());
