/**
 * @module ctt
 * @submodule ctt-ad
 */
dixim.namespace('dixim.ctt.ad');

/**
 * @module ctt-ad
 * @submodule deletion-dialog-controller
 */

(function() {
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var DeletionDialogView = dixim.ctt.ad.DeletionDialogView;
var ContentsTransferApiClient = dixim.ctt.api.ContentsTransferApiClient;
var _ = dixim.ctt.ResourceBundle.get;
var ErrorDialog = dixim.ctt.ErrorDialog;

var DeletionDialogController;
var prototype;

/**
 * @class DeletionDialogController
 * @constructor
 * @param {Object} configs
 * @param {Object} configs.targetItems
 * @param {ContentsTransferApiClient} [configs.model]
 */
DeletionDialogController = function(configs) {
  EventTarget.call(this);
  this._initialize(configs);
};
dixim.inherit(DeletionDialogController, EventTarget);
dixim.ctt.ad.DeletionDialogController = DeletionDialogController;
prototype = DeletionDialogController.prototype;

/**
 * Triggered when the deletion has been done.
 *
 * @event close
 * @param {Event} event
 * @param {Object} event.args
 * @param {Object} event.args.deleted true if the deletion is completed, otherwise false.
 */
var CLOSE = 'close';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 * @param {Object} configs.targetItems
 * @param {ContentsTransferApiClient} [configs.model]
 */
prototype._initialize = function(configs) {
  configs = configs || {};

  this._model = configs.model || new ContentsTransferApiClient();
  this._targetItems = configs.targetItems || null;
  this._view = new DeletionDialogView(configs);
  this._session = null;
  this._queue = [];
  this._deleted = false;
  this._didFinishedFunction = configs.finish || null;
};

/**
 * @method start
 */
prototype.start = function() {
  var v = this._view;
  var m = this._model;

  v.addEventListener('delete', this._didClickDelete, this);
  v.addEventListener('cancel', this._didClickCancel, this);

  m.addEventListener('remove', this._didRemove, this);
  m.addEventListener('removeError', this._didFailRemove, this);

  v.show();
};

prototype._didClickDelete = function() {
  if (this._queue.length) return;

  this._view.showActivityIndicator();

  this._queue = this._targetItems.concat();
  this._deleted = true;
  this._processQueue();
};

prototype._processQueue = function() {
  var item;
  var total = this._targetItems.length;
  var queue = this._queue;
  var query = {};

  if (!queue.length) {
    this._didFinishDeletion();
    return;
  }

  item = queue.shift();
  item.title = item.srcContainerTitle + " (" + item.srcFriendlyName + ")";
  this._view.updateProgress(item, total - queue.length - 1, total);

  query.taskId = item.taskId;
  query.success = dixim.proxy(function() {
    this._didRemove();
  }, this);

  query.error = dixim.proxy(function(e) {
    var errorCode = e.args.errorCode;
    var error_description = _(["ctt_ad_error_",errorCode].join(''));

    if (error_description.substr(0,13) === "[ctt_ad_error_") {
      error_description = _("ctt_ad_error_other");
    }
    this._didFailRemove({title: _('ctt_deletion_view_title'), description: error_description});
  }, this);
  this._session = this._model.deleteTask(query);
};

prototype._didRemove = function() {
  this._session = null;
  this._processQueue();
};

prototype._didFailRemove = function(error) {
  this._view.hideActivityIndicator();
  this._session = null;
  this._queue = [];
  this._view.showError(error);
};

prototype._didClickCancel = function() {
  if (!this._queue.length && !!!this._session) {
    this._view.destroy();
    this.dispatchEvent(new Event(CLOSE, {deleted: this._deleted}));
    return;
  }
  this._queue = [];
};

prototype._didFinishDeletion = function() {
  this._view.hideActivityIndicator();
  this._view.destroy();
  this.dispatchEvent(new Event(CLOSE, {deleted: this._deleted}));
  if (typeof this._didFinishedFunction === "function") {
    this._didFinishedFunction();
  }
};

/**
 * @method stop
 */
prototype.stop = function() {
  this._view.hide();
};

/**
 * @method destroy
 */
prototype.destroy = function() {
  this._view.destroy();
};

/**
 * @method _showErrorDialog
 * @private
 * @param {String} error error ID.
 */
prototype._showErrorDialog = function(error) {
  var dialog;
  var query = {type: error};

  if (!!this._modal) {
    this._modal.destroy();
    this._modal = null;
  }
  if (typeof error.title !== "undefined") {
    query.title = error.title;
  }
  if (typeof error.description !== "undefined") {
    query.description = error.description;
  }

  dialog = new ErrorDialog(query);
  dialog.addEventListener('close', this._didCloseModalDialog, this);

  dialog.show();
  this._modal = dialog;
};

prototype._didCloseModalDialog = function(e) {
  var args = e.args;

  if (this._modal === args.dialog) {
    this._modal.destroy();
    this._modal = null;
  }
};

}());
