/**
 * @module ctt
 * @submodule renaming-controller
 */
(function() {
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var RenamingView = dixim.ctt.RenamingView;
var ContentsTransferApiClient = dixim.ctt.api.ContentsTransferApiClient;
var _ = dixim.ctt.ResourceBundle.get;

var RenamingController;
var prototype;

/**
 * @class RenamingController
 * @constructor
 * @param {Object} configs
 * @param {ContentsTransferApiClient} [configs.model]
 */
RenamingController = function(configs) {
  EventTarget.call(this);
  this._initialize(configs);
};
dixim.inherit(RenamingController, EventTarget);
dixim.ctt.RenamingController = RenamingController;
prototype = RenamingController.prototype;

/**
 * Triggered when the rename has been done.
 *
 * @event close
 * @param {Event} event
 * @param {Object} event.args
 * @param {Object} event.args.renamed true if the renaming is completed, otherwise false.
 */
var CLOSE = 'close';

/**
 * @class ErrorCode
 * @static
 * @for RenamingController
 */
var ErrorCode =  {
  /**
   * @property EMPTY_STRING
   * @default 'empty'
   * @final
   * @static
   */
  EMPTY_STRING: 'empty',
  /**
   * @property FORBIDDEN
   * @default 406
   * @final
   * @static
   */
  FORBIDDEN: 406,
  /**
   * @property CONFLICTED
   * @default 409
   * @final
   * @static
   */
  CONFLICTED: 409,
  /**
   * @property EXCEEDS_MAX_LENGTH
   * @default 413
   * @final
   * @static
   */
  EXCEEDS_MAX_LENGTH: 413,
  /**
   * @property IN_PROGRESS
   * @default 513
   * @final
   * @static
   */
  IN_PROGRESS: 513
};

var ErrorMessage = (function() {
  var ret = {};
  var i = null;
  var code;
  var text;

  for (i in ErrorCode) {
    if (!ErrorCode.hasOwnProperty(i)) continue;
    code = ErrorCode[i];
    text = _(['ctt_renaming_view_error_', code].join(''));
    ret[code] = text;
  }
  return ret;
}());

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 * @param {Object} configs.targetItem
 * @param {ContentsTransferApiClient} [configs.model]
 */
prototype._initialize = function(configs) {
  configs = configs || {};

  this._model = configs.model || new ContentsTransferApiClient();
  this._targetItem = configs.targetItem;
  this._view = new RenamingView();
  this._session = null;
};

/**
 * @method start
 */
prototype.start = function() {
  var v = this._view;
  var m = this._model;

  v.addEventListener('render', this._didRenderView, this);
  v.addEventListener('rename', this._didClickRename, this);
  v.addEventListener('cancel', this._didClickCancel, this);

  m.addEventListener('rename', this._didRename, this);
  m.addEventListener('renameError', this._didFailRename, this);

  v.show();
};

prototype._didRenderView = function(e) {
  this._view.updateTargetItem(this._targetItem);
};

prototype._validateName = function(name) {
  var getStringSize = function(str) {
    // Matches only the 10.. bytes that are non-initial characters in a multi-byte sequence.
    var ret = encodeURIComponent(str).match(/%[89ABab]/g);
    return str.length + (ret ? ret.length : 0);
  };

  if (!!!name) {
    return ErrorCode.EMPTY_STRING;
  } else if (getStringSize(name) > 255) {
    return ErrorCode.EXCEEDS_MAX_LENGTH;
  };

  return null;
};

prototype._didClickRename = function(e) {
  var args = e.args;
  var newName = args.newName;
  var item = args.targetItem;
  var error;

  if (!!this._session) return;

  if (!!(error = this._validateName(newName))) {
    this._view.showError(ErrorMessage[error]);
    return;
  }
  this._view.showActivityIndicator();
  this._session = this._model.rename({id: item.id, newName: newName});
};

prototype._didRename = function(e) {
  this._view.destroy();
  this.dispatchEvent(new Event(CLOSE, {renamed: true}));
  this._session = null;
};

prototype._didFailRename = function(e) {
  var code = e.args.errorCode;

  this._view.hideActivityIndicator();
  this._view.showError(ErrorMessage[code] || _('ctt_renaming_view_error_unknown'));
  this._session = null;
};

prototype._didClickCancel = function(e) {
  if (!!this._session) return;

  this._view.destroy();
  this.dispatchEvent(new Event(CLOSE, {renamed: false}));
};

/**
 * @method stop
 */
prototype.stop = function() {
  this._view.hide();
};

/**
 * @method destroy
 */
prototype.destroy = function() {
  this._view.destroy();
};

}());
