/**
 * @module dixim
 * @submodule common
 */
dixim.namespace('dixim.common');

/**
 * @module common
 * @submodule api-client-auto-download
 */
(function() {
var ApiClientAutoDownload;
var prototype;
var suprtConstructor;
var ApiClient = dixim.common.ApiClient;
var Configs = dixim.ctt.Configs;

/**
 * Provides interfaces to download automatically API.
 *
 * This API client supports two type of event handling method,
 * callback and event-subscription.
 *
 * The callback method can be specified as the parameter of each method.
 * (Such as request.success, request.error or request.complete.)
 * When the callback is called, an Event instance which is related to
 * the triggered method is passed to the callback.
 *
 * This API client is also an EventTarget, you can subscribe an event
 * by addEventListener().
 *
 * @class ApiClientAutoDownload
 * @constructor
 * @extends EventTarget
 * @param {Object} [configs]
 */
ApiClientAutoDownload = function(configs) {
    ApiClient.call(this, configs);
};

dixim.inherit(ApiClientAutoDownload, ApiClient);
dixim.common.ApiClientAutoDownload = ApiClientAutoDownload;
prototype = ApiClientAutoDownload.prototype;
superConstructor = ApiClientAutoDownload.suprtConstructor;

/**
 * Fetch source list.
 *
 * @method fetchSrcList
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchSrcList = function(request) {
    return this.doSimpleFetch.call(this, 'fetchSrcList', [Configs.get('apiAds'), '/src/list'].join(''), request);
};

/**
 * Browse source server.
 *
 * @method browseSrc
 * @param {Object} request
 * @param {String} [request.udn]
 * @param {String} [request.objectId]
 * @param {String} [request.browseFlag]
 * @param {String} [request.startingIndex]
 * @param {String} [request.requestedCount]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.browseSrc = function(request) {
    var query = {};

    query.udn = request.udn;
    query.objectId = request.objectId;
    query.browseFlag = request.browseFlag || 'BrowseDirectChildren';
    query.startingIndex = request.startingIndex || 0;
    query.requestedCount = request.requestedCount || 0;

    return this.doSimpleMod.call(this, 'browseSrc', [Configs.get('apiAds'), '/src/browse'].join(''), query, request);
};

/**
 * Fetch dest storage list.
 *
 * @method fetchDstStorageList
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchDstStorageList = function(request) {
    return this.doSimpleFetch.call(this, 'fetchDstStorageList', [Configs.get('apiAds'), '/dst/storage/list'].join(''), request);
};

/**
 * Browse dest server.
 *
 * @method browseDst
 * @param {Object} request
 * @param {String} [request.objectId]
 * @param {String} [request.browseFlag]
 * @param {String} [request.startingIndex]
 * @param {String} [request.requestedCount]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.browseDst = function(request) {
    var query = {};

    query.objectId = request.objectId;
    query.browseFlag = request.browseFlag;
    query.startingIndex = request.startingIndex || 0;
    query.requestedCount = request.requestedCount || 0;

    return this.doSimpleMod.call(this, 'browseDst', [Configs.get('apiAds'), '/dst/browse'].join(''), query, request);
};

/**
 * Set task.
 *
 * @method setTask
 * @param {Object} request
 * @param {String} [request.srcUdn]
 * @param {String} [request.srcContainerId]
 * @param {String} [request.dstContainerId]
 * @param {String} [request.dstContainerPath]
 * @param {Boolean} [request.sdFirst]
 * @param {String} [request.reservationType]
 * @param {String} [request.timedReservationStartTime]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.setTask = function(request) {
    var query = {};

    query.srcUdn = request.srcUdn;
    query.srcContainerId = request.srcContainerId;
    if (typeof request.dstContainerId !== 'undefined') {
        query.dstContainerId = request.dstContainerId;
    } else {
        query.dstContainerPath = request.dstContainerPath;
    }
    query.sdFirst = request.sdFirst;
    query.reservationType = request.reservationType;
    if (typeof request.timedReservationStartTime !== 'undefined') query.timedReservationStartTime = request.timedReservationStartTime;

    return this.doSimpleMod.call(this, 'setTask', [Configs.get('apiAds'), '/task/add'].join(''), query, request);
};

/**
 * Remove task.
 *
 * @method removeTask
 * @param {Object} request
 * @param {String} [request.taskId]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.removeTask = function(request) {
    var query = {};

    query.taskId = request.taskId;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'removeTask', [Configs.get('apiAds'), '/task/del'].join(''), query, request);
};

/**
 * Change task.
 *
 * @method changeTask
 * @param {Object} request
 * @param {String} [request.taskId]
 * @param {Boolean} [request.sdFirst]
 * @param {String} [request.reservationType]
 * @param {String} [request.timedReservationStartTime]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.changeTask = function(request) {
    var query = {};

    query.taskId = request.taskId;
    if (typeof request.sdFirst !== 'undefined') query.sdFirst = request.sdFirst;
    if (typeof request.reservationType !== 'undefined') query.reservationType = request.reservationType;
    if (typeof request.timedReservationStartTime !== 'undefined') query.timedReservationStartTime = request.timedReservationStartTime;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'changeTask', [Configs.get('apiAds'), '/task/mod'].join(''), query, request);
};

/**
 * Fetch task list.
 *
 * @method fetchTaskList
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchTaskList = function(request) {
    return this.doSimpleFetch.call(this, 'fetchTaskList', [Configs.get('apiAds'), '/task/list'].join(''), request);
};

/**
 * Fetch item list.
 *
 * @method fetchItemList
 * @param {Object} request
 * @param {Integer[]} [request.taskIds]
 * @param {String} [request.filter]
 * @param {String} [request.startingIndex]
 * @param {String} [request.requestedCount]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchItemList = function(request) {
    var query = {};

    request = request || {};
    if (typeof request.taskIds !== 'undefined') query.taskIds = request.taskIds.join(',');
    if (typeof request.filter !== 'undefined') query.filter = request.filter;
    if (typeof request.startingIndex !== 'undefined') query.startingIndex = request.startingIndex;
    if (typeof request.requestedCount !== 'undefined') query.requestedCount = request.requestedCount;

    return this.doSimpleMod.call(this, 'fetchItemList', [Configs.get('apiAds'), '/item/list'].join(''), query, request);
};

/**
 * Remove item.
 *
 * @method removeItem
 * @param {Object} request
 * @param {Integer} [request.taskId]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.removeItem = function(request) {
    var query = {};

    request = request || {};
    if (typeof request.taskId !== 'undefined') query.taskId = request.taskId;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'removeItem', [Configs.get('apiAds'), '/item/del'].join(''), query, request);
};

/**
 * Change item status.
 *
 * @method changeItemStatus
 * @param {Object} request
 * @param {Integer} [request.itemId]
 * @param {Integer} [request.state]
 * @param {Integer} [request.errorCode]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.changeItemStatus = function(request) {
    var query = {};

    query.itemId = request.itemId;
    query.state = request.state;
    if (typeof request.errorCode !== 'undefined') query.errorCode = request.errorCode;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'changeItemStatus', [Configs.get('apiAds'), '/settings/item/state/mod'].join(''), query, request);
};

/**
 * Fetch space.
 *
 * @method fetchSpace
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchSpace = function(request) {
    return this.doSimpleFetch.call(this, 'fetchSpace', [Configs.get('apiAds'), '/settings/space/get'].join(''), request);
};

/**
 * Set space.
 *
 * @method setSpace
 * @param {Object} request
 * @param {Integer} [request.size]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.setSpace = function(request) {
    var query = {};

    query.size = request.size;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'setSpace', [Configs.get('apiAds'), '/settings/space/set'].join(''), query, request);
};

/**
 * Fetch download move setting.
 *
 * @method fetchDownloadMoveSetting
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchDownloadMoveSetting = function(request) {
    return this.doSimpleFetch.call(this, 'fetchDownloadMoveSetting', [Configs.get('apiAds'), '/settings/move/get'].join(''), request);
};

/**
 * Set download move setting.
 *
 * @method setDownloadMoveSetting
 * @param {Object} request
 * @param {Boolean} [request.allowMove]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.setDownloadMoveSetting = function(request) {
    var query = {};

    query.allowMove = request.allowMove;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'setDownloadMoveSetting', [Configs.get('apiAds'), '/settings/move/set'].join(''), query, request);
};

/**
 * Fetch filter list.
 *
 * @method fetchFilterList
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchFilterList = function(request) {
    return this.doSimpleFetch.call(this, 'fetchFilterList', [Configs.get('apiAds'), '/filter/list'].join(''), request);
};

/**
 * Set filter.
 *
 * @method setFilter
 * @param {Object} request
 * @param {String} [request.filterName]
 * @param {String} [request.combCond]
 * @param {Integer[]} [request.taskIds]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.setFilter = function(request) {
    var query = {};

    query.filterName = request.filterName;
    query.combCond = request.combCond;
    if (typeof request.taskIds !== 'undefined') query.taskIds = request.taskIds.join(',');

    return this.doSimpleMod.call(this, 'setFilter', [Configs.get('apiAds'), '/filter/add'].join(''), query, request);
};

/**
 * Remove filter.
 *
 * @method removeFilter
 * @param {Object} request
 * @param {Integer} [request.filterId]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.removeFilter = function(request) {
    var query = {};

    query.filterId = request.filterId;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'removeFilter', [Configs.get('apiAds'), '/filter/del'].join(''), query, request);
};

/**
 * Change filter name.
 *
 * @method changeFilterName
 * @param {Object} request
 * @param {Integer} [request.filterId]
 * @param {String} [request.name]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.changeFilterName = function(request) {
    var query = {};

    query.filterId = request.filterId;
    query.name = request.name;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'changeFilterName', [Configs.get('apiAds'), '/filter/name/mod'].join(''), query, request);
};

/**
 * Change combination.
 *
 * @method changeFilterComb
 * @param {Object} request
 * @param {Integer} [request.filterId]
 * @param {String} [request.combCond]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.changeFilterComb = function(request) {
    var query = {};

    query.filterId = request.filterId;
    query.combCond = request.combCond;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'changeFilterComb', [Configs.get('apiAds'), '/filter/comb/mod'].join(''), query, request);
};

/**
 * Link task filter.
 *
 * @method linkTaskFilter
 * @param {Object} request
 * @param {Integer} [request.taskId]
 * @param {Integer} [request.filterId]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.linkTaskFilter = function(request) {
    var query = {};

    query.taskId = request.taskId;
    query.filterId = request.filterId;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'linkTaskFilter', [Configs.get('apiAds'), '/filter/link'].join(''), query, request);
};

/**
 * Unlink task filter.
 *
 * @method unlinkTaskFilter
 * @param {Object} request
 * @param {Integer} [request.taskId]
 * @param {Integer} [request.filterId]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.unlinkTaskFilter = function(request) {
    var query = {};

    query.taskId = request.taskId;
    query.filterId = request.filterId;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'unlinkTaskFilter', [Configs.get('apiAds'), '/filter/unlink'].join(''), query, request);
};

/**
 * Fetch cond list.
 *
 * @method fetchCondList
 * @param {Object} request
 * @param {Integer} [request.filterId]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchCondList = function(request) {
    var query = {};

    query.filterId = request.filterId;

    return this.doSimpleMod.call(this, 'fetchCondList', [Configs.get('apiAds'), '/cond/list'].join(''), query, request);
};

/**
 * Set cond.
 *
 * @method setCond
 * @param {Object} request
 * @param {Integer} [request.filterId]
 * @param {Integer} [request.name]
 * @param {Integer} [request.op]
 * @param {String} [request.value]
 * @param {String} [request.value2]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.setCond = function(request) {
    var query = {};

    query.filterId = request.filterId;
    query.name = request.name;
    query.op = request.op;
    query.value = request.value;
    query.value2 = request.value2;

    return this.doSimpleMod.call(this, 'setCond', [Configs.get('apiAds'), '/cond/add'].join(''), query, request);
};

/**
 * Remove cond.
 *
 * @method setCond
 * @param {Object} request
 * @param {Integer} [request.condId]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.removeCond = function(request) {
    var query = {};

    query.condId = request.condId;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'removeCond', [Configs.get('apiAds'), '/cond/del'].join(''), query, request);
};

/**
 * Change cond.
 *
 * @method changeCond
 * @param {Object} request
 * @param {Integer} [request.condId]
 * @param {Integer} [request.name]
 * @param {Integer} [request.op]
 * @param {String} [request.value]
 * @param {String} [request.value2]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.changeCond = function(request) {
    var query = {};

    query.condId = request.condId;
    query.name = request.name;
    query.op = request.op;
    query.value = request.value;
    if (typeof request.value2 !== 'undefined') query.value2 = request.value2;

    return this.doSimpleMod.call(this, 'changeCond', [Configs.get('apiAds'), '/cond/mod'].join(''), query, request);
};

}());
