/**
 * @module dixim
 * @submodule contents
 */
dixim.namespace('dixim.contents');

/**
 * @module contents
 * @submodule deletion-dialog-controller
 */

(function() {
var DeletionDialogController;
var prototype;
var superConstructor;
var DeletionDialogView = dixim.contents.DeletionDialogView;
var EventTarget = dixim.event.EventTarget;
var Event = dixim.event.Event;
var _ = dixim.ctt.ResourceBundle.get;
var ApiClientTransfer = dixim.common.ApiClientTransfer;
var ErrorModalDialogView = dixim.common.ErrorModalDialogView;
var Enum = dixim.ctt.util.Enum;
var Decoder = dixim.common.Decoder;

/**
 * @class DeletionDialogController
 * @constructor
 * @param {Object} configs
 */
DeletionDialogController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(DeletionDialogController, EventTarget);
dixim.contents.DeletionDialogController = DeletionDialogController;
prototype = DeletionDialogController.prototype;
superConstructor = DeletionDialogController.superConstructor;

var YES = 'yes';
var COMPLETE = 'complete';
var CLOSE = 'close';
var UPDATE = 'update';

var Mode = Enum.fetch('DELETION');

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    configs.id = configs.id || '';
    this._id = [configs.id, 'deletion-dialog'].join('');

    this._decoder = (typeof Decoder !== 'undefined') ? new Decoder() : null;
    this._contents = configs.contents;
    this._isProcessError;
    this._mode = configs.mode || Mode.CONTENTS;

    this._model = new ApiClientTransfer();
    this._model.addEventListener('remove', this._successDelete, this);
    this._model.addEventListener('removeError', this._errorDelete, this);
    this._model.addEventListener('removeDms', this._successDelete, this);
    this._model.addEventListener('removeDmsError', this._errorDelete, this);
    this._model.addEventListener('removeReservation', this._successDelete, this);
    this._model.addEventListener('removeReservationError', this._errorDelete, this);

    this._view = new DeletionDialogView(configs);
    this._view.addEventListener(YES, this._clickYes, this);
    this._view.addEventListener(COMPLETE, this._complete, this);
};

/**
 * @method start
 */
prototype.start = function() {
    this._view.show();
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype._successDelete = function() {
    this._view.decrementDescription();
};

prototype._errorDelete = function() {
    this._isProcessError = true;
    this._view.decrementDescription();
};

prototype._deleteContents = function() {
    var contentsLen = this._contents.length;
    var query = {};

    this._isProcessError = false;
    for (var i = 0; i < contentsLen; i += 1) {
        switch(this._mode) {
            case Mode.CONTENTS:
                this._remove(this._contents[i].id);
                break;
            case Mode.DMS:
                query.udn = this._contents[i].udn;
                this._model.removeDms(query);
                break;
            case Mode.LOG:
                query.id = this._contents[i].id;
                this._model.removeReservation(query);
                break;
        }
    }
};

prototype._remove = function(id) {
    var query = {};

    query.success = $.proxy(function(e) {
        var token = e.args.token;
        var bufArr = this._decoder.base64ToArrayBuffer(token);
        var rBufArr;
        var query = {};

        this._decoder.setData(bufArr);
        this._decoder.decode();
        rBufArr = this._decoder.getData();

        query.id = id;
        query.token = this._decoder.base64FromArrayBuffer(rBufArr);
        this._session = this._model.remove(query);
    }, this);
    query.error = $.proxy(function(e) {
        var errorCode = e.args.errorCode;
        var query = {};

        if (errorCode === 404) {
            query.id = id;
            this._session = this._model.remove(query);
        } else {
            this._isProcessError = true;
            this._view.decrementDescription();
        }
    }, this);

    this._model.getToken(query);
};

prototype._complete = function() {
    this.stop();
    if (this._isProcessError) {
        this._errorDialog(_('contents_deletion_dialog_error'));
        this.dispatchEvent(new Event(CLOSE));
    } else {
        this.dispatchEvent(new Event(UPDATE));
    }
};

prototype._clickYes = function() {
    this._deleteContents();
};

prototype._errorDialog = function(text) {
    var query = {};
    var dialog;

    switch(this._mode) {
        case Mode.CONTENTS:
            query.title = _('contents_operation_menu_delete');
            break;
        case Mode.DMS:
            query.title = _('contents_transfer_wizard_deletion_title');
            break;
        case Mode.LOG:
            query.title = _('contents_transfer_reservation_dialog_delete_error_description');
            break;
    }

    query.id = this._id;
    query.doubleDialog = true;
    query.description = text;
    dialog = new ErrorModalDialogView(query);
    dialog.show();
};

}());
